/**
 * This is the first interation of the ColorArea function that can select a rectangular 
 * regions being drawn to the screeen and tell the user what the average color is, along with
 * the threshold of the color (the flux, if you will)
 * One needs to be able to 1) Set the area to search for
 *                         2) Get the average color of that area
 *                         3) Get the threshold of each color and an overall threshold (?) 
 */

class ColorArea{
  private color outlinecol = color(255, 80);
  private color fillcol = color(0, 30);
  private int startx;            //the starting location of the rect
  private int starty;            //the starting locatioin of the rect
  private int endy;              //the ending location of the rect
  private int endx;              //the ending location of the rect
  private color averageColor;    //the average color found in the last iteration
  private float threshold;       //the threshold between the average color and the maximum

  /**
   * Defualt Contructors and so on
   */
  public ColorArea(int x1, int y1, int x2, int y2){
    this.startx = x1;
    this.starty = y1;
    this.endx = x2;
    this.endy = y2;

    this.averageColor = color(0);
    this.threshold = 0;
  }

  /**
   * draw functionality, fool!
   * draws a nice transporent recttangle around the area being selected
   */
  public void draw(){
    fill(this.averageColor);
    //noFill();
    stroke(outlinecol);

    //draw the outline box to show where the color is being averaged from
    rect(this.startx, this.starty, this.endx - this.startx, this.endy - this.starty);

    //and show some fancy text show what the color actually is
    fill(color(255));
    text("(" + (this.averageColor >> 16 & 0xFF) + ", " + (this.averageColor >> 8 & 0xFF) + ", " + (this.averageColor & 0xFF) + ")", this.startx, this.endy + 12);
  }

  /**
   * Captures the colors and does the processing on them
   */
  public void captureColor(PImage img){
    //loadPixels();      //pull in the current pixels into the system
    
    //ratios to match image size to screen size
    float ratio = float(img.width) / float(width);
    
    //modify start indices by the ratio
    int xScalledStart = int(this.startx * ratio);
    int xScalledEnd = int(this.endx * ratio);
    int yScalledStart = int(this.starty * ratio);
    int yScalledEnd = int(this.endy * ratio);

    //setup the constants, using some good defualt;
    int redTotal = 0;
    int redMax = 0;
    int redMin = 255;
    int greenTotal = 0;
    int greenMax = 0;
    int greenMin = 255;
    int blueTotal = 0;
    int blueMax = 0;
    int blueMin = 255;

    //some variable for legiblility
    color currentColor;
    int currentRed;
    int currentGreen;
    int currentBlue;
    int count = 0;

    //loop through the pixels area and extract color info
    for(int i = xScalledStart; i < xScalledEnd; i++){
      for(int j = yScalledStart; j < yScalledEnd; j++){
        //fast extract color!
        currentColor = img.pixels[j * img.width + i];
        currentRed = currentColor >> 16 & 0xFF;
        currentGreen = currentColor >> 8 & 0xFF;
        currentBlue = currentColor & 0xFF;

        //now that the color has been extracted, add into the running sum
        redTotal += currentRed;
        greenTotal += currentGreen;
        blueTotal += currentBlue;

        //and find the max / mins;
        if(currentRed < redMin){
          redMin = currentRed;
        }
        if(currentRed > redMax){
          redMax = currentRed;
        }
        if(currentGreen < greenMin){
          greenMin = currentGreen;
        }
        if(currentGreen > greenMax){
          greenMax = currentGreen;
        }
        if(currentBlue < blueMin){
          blueMin = currentBlue;
        }
        if(currentBlue > blueMax){
          blueMax = currentBlue;
        }

        //all done, count up the count
        count++;
      }
    }
    this.averageColor = color(redTotal / count, greenTotal / count, blueTotal / count);
  }

  //
  //GETTERS AND SETTER
  //

  public void setArea(int x1, int y1, int x2, int y2){
    this.startx = x1;
    this.starty = y1;
    this.endx = x2;
    this.endy = y2;
  }
  public color getAverageColor(){
    return this.averageColor; 
  }
  public float getAverageLuminosity(){
    return brightness(this.averageColor) / 255.0;
  }
}
