/**
 * A console is a combination of a textfield and textbox
 * that allows some nice text interaction with a system
 */

class Console{

  private ArrayList interactors;      //every single ConsoleInteractor who likes this guy
  private TextTag title;              //the title and name of the console
  private TextBox textbox;            //The text box used for showing the history of the input
  private TextField textfield;        //The text field used for input  
  private boolean bIsActive;          //True when this console is interacting with the user
  private boolean bIsVisible;         //True when the console should be shown, else, not.

  /**
   * defualt constructor
   */

  public Console(PFont font, Point loc, int lineCount, int lineSpacing, int bwidth){
    interactors = new ArrayList();
    title = new TextTag(font, new Point(loc.x, loc.y - (lineSpacing * 2) - 2), 100, lineSpacing);
    title.setText("Puppet Console");
    textbox = new TextBox("ConsoleTextBox", font, lineCount, lineSpacing, loc, bwidth);
    textfield = new TextField(font, new Point(loc.x, loc.y + (lineSpacing * lineCount) + 12), bwidth, lineSpacing); 
    this.bIsActive = false;
  }

  /**
   * Draw!
   */
  public void draw(){
    this.title.draw();
    this.textbox.draw();
    this.textfield.draw();
  }

  /**
   * Tick this shite!
   */
  public void tick(float delta){

  }

  /**
   * Add a interactor
   */
  public void addInteractor(ConsoleInteractor ci){
    this.interactors.add(ci);
  }

  /**
   * And easily remove them to
   */
  public void removeInteractor(ConsoleInteractor ci){
    this.interactors.remove(ci);
  }

  /**
   * Key event! Lets us process stuff!
   */
  public void keyEvent(char newChar){
    //if active send the key event to the console
    if(this.bIsActive){
      this.textfield.addCharacter(newChar);
    }

    //if its the 'enter' key (raw ascii 10) add the current line to the text box and clear the field
    if(newChar == 10){
      this.addLine(this.textfield.getText());

      //before we clear the text buffer in the input field, send the text to all interactors
      ConsoleInteractor ci;
      for(int i = 0; i < this.interactors.size(); i++){
        ci = (ConsoleInteractor)(this.interactors.get(i));
        ci.consoleEvent(this.textfield.getText());
      }

      //purge the text!
      this.textfield.setText("");
    }
  }

  /**
   * Adds text right to the console
   */
  public void addLine(String newLine){
    this.textbox.addLine("> " + newLine); 
  }

  /**
   * GETTERS AND SETTERS
   */
  public void setActive(boolean b){
    this.bIsActive = b;
    if(this.bIsActive){
      //if the console is now becomming active, set the boxes to have focus
      this.textfield.setFocus(true);
    } 
    else {
      //or to lose it
      this.textfield.setFocus(false);
    }
  }
  public boolean isActive(){
    return this.bIsActive;
  }
}

/**
* A console interactor is a nice little interface that classes
* should implement if they want to recieve events from a console
*/
interface ConsoleInteractor{
  
  /**
  * This sends the literal String to a ConsoleInteractor
  */
  public void consoleEvent(String input);
}
