class UIElement{

  public final int gutter = 5; //gutter amount around the text
  public final color backing = color(0, 40);

  /**
   * UIElement members
   */

  private String name;             //a basic identifying character string
  private Point location;          //a location that the element exsists at
  private boolean bHasFocus;       //true if the UI element should be interacting with the user
  private int boxWidth;            //draw Width
  private int boxHeight;           //draw height
  private ArrayList interactors;   //a list of interactors that want to get some clicks! all should be UIInteractors

  /**
   * Default constructor
   */
  public UIElement(){
    this("noname", new Point(), 0, 0);
  }

  public UIElement(String newname, Point newlocation, int bwidth, int bheight){
    this.name = newname;
    this.location = newlocation;
    this.bHasFocus = false;
    this.boxWidth = bwidth;
    this.boxHeight = bheight;
    this.interactors = new ArrayList();
    this.bHasFocus = false;
  }

  /**
   * Core draw statement!
   */
  public void draw(){ 
    //draw the background transparent box first, to frame in the window
    stroke(0, 0, 255);
    fill(backing);
    //rect(this.location.x - gutter, this.location.y - gutter, this.boxWidth, this.boxHeight);
    //rect(this.location.x, this.location.y - this.boxHeight, this.boxWidth, this.boxHeight);
  }

  /**
   * Core text statement, for processing animations and such
   */
  public void tick(float delta){

  }

  /**
   * GETTERS AND SETTERS
   */
  public String getName(){
    return this.name;
  }
  public Point getLocation(){
    return this.location;
  }
  public boolean bHasFocus(){
    return this.bHasFocus; 
  }
  public int getWidth(){
    return this.boxWidth;
  }
  public int getHeight(){
    return this.boxHeight;
  }

  public void setName(String newname){
    this.name = newname;
  }
  public void setLocation(Point newlocation){
    //this.location.setTo(newlocation); 
  }
  public void setLocation(int x, int y){
    //this.location.setTo(x, y, 0); 
  }
  public void setFocus(boolean newFocus){
    this.bHasFocus = newFocus;
  }
  public boolean isHit(Point p){
    if(this.location.x < p.x && this.location.x + this.boxWidth > p.x){
      if(this.location.y - this.boxHeight < p.y && this.location.y > p.y){
        return true;
      }
    }
    return false;
  }
}
/**
 * The text tag is the most basic text display in the UIElements toolkit, simply
 * just a line of text, c'est tout.
 */

class TextTag extends UIElement{
  private String characters;    //the characters that are being drawn on the screen,
  private PFont font;           //the font being used to draw with, this is used directly, without scaling


  /**
   * default contructor
   */
  public TextTag(){
    super();
    this.characters = "";
    this.font = null;
  }

  public TextTag(PFont thefont){
    this();
    this.font = thefont;
  }

  public TextTag(PFont thefont, Point loc, int bwidth, int bheight){
    super("noname", loc, bwidth, bheight);
    this.font = thefont;
    this.characters = "";
  }

  /**
   * Draw this shite
   */
  public void draw(){
    super.draw();      //draw the super

    if(font != null && !this.characters.equals("")){
      //we are good to draw so
      //set the font:      
      textFont(this.font);
      fill(0);

      //and draw:
      pushMatrix();
      //translate(super.getLocation().x, super.getLocation().y + super.getHeight() / 2);
      translate(super.getLocation().x, super.getLocation().y);
      text(this.characters, 0, 0);
      popMatrix();
    }
  }

  /**
   * Tick this shite
   */
  public void tick(float delta){
    super.tick(delta);    //tick that super.
  }
  /**
   * set the text
   */
  public void setText(String tex){
    this.characters = tex;
  }
  /**
   * get text
   */
  public String getText(){
    return this.characters;
  }
  public void setFont(PFont f){
    this.font = f;
  }
  public PFont getFont(){
    return this.font;
  }
}

/**
 * The text field class holds and displays keyboard input that it is sent via the
 * add character methods and such
 */

class TextField extends TextTag{
  private float ratio;          //the scale ratio, usually just plain old 1
  private int lengthLimit;      //maximum number of characters


  /**
   * default contructor
   */
  public TextField(){
    super();
    this.ratio = 1;
    this.lengthLimit = -1;
  }

  public TextField(PFont thefont){
    super(thefont);
    this.ratio = 1;
    this.lengthLimit = -1;
  }

  public TextField(PFont thefont, Point loc, int bwidth, int bheight){
    super(thefont, loc, bwidth, bheight);
    this.lengthLimit = -1;
  }

  public TextField(PFont thefont, Point loc, float rat, int bwidth, int bheight){
    this(thefont, loc, bwidth, bheight);
    this.ratio = rat;
  }

  /**
   * Draw this shite
   */
  public void draw(){
    super.draw();      //draw the super
    /*
    if(super.getFont() != null && !super.getText().equals("")){
     //we are good to draw so
     //set the font:      
     textFont(super.getFont());
     fill(color(250));
     
     //and draw:
     pushMatrix();
     translate(super.getLocation().x, super.getLocation().y + super.getHeight() / 2);
     scale(this.ratio);
     text(super.getText(), 0, 0);
     popMatrix();
     }
     */
  }

  /**
   * Tick this shite
   */
  public void tick(float delta){
    super.tick(delta);    //tick that super.
  }

  /**
   * Adds a character to the end of a string
   */
  public void addCharacter(char newChar){
    //limit to valid text input
    if(super.bHasFocus() && (this.lengthLimit == -1 || super.getText().length() < this.lengthLimit)){
      if(newChar >= ' ' && newChar <= '~' && newChar != '`'){
        super.setText(super.getText() + newChar);
      }
    }

    if(super.bHasFocus() && key == 8 && super.getText().length() > 0){
      super.setText(super.getText().substring(0, super.getText().length()-1));
    }
  }

  /**
   * set the legth limit
   */
  public void setLengthLimit(int le){
    this.lengthLimit = le;
  }
}


/**
 * TextBox is a UI Element that can show one or more lines of text.
 * You can just set text from the start, or add text as it comes, 
 * and textbox will move the text up as needed.
 */

class TextBox extends UIElement{


  private String[] lines;      //the lines of text in the box, size = number of lines on screen
  private int lineSpacing;   //pixel space between lines;
  private int maxLineCount;    //maximum lines in the text box
  private PFont font;          //the font to use, yo!


  /**
   * default contructor
   */
  public TextBox(String name, PFont newfont, int lineCount, int lineSpace, Point loc, int bwidth){
    super(name, loc, bwidth, (lineCount * lineSpace));
    this.maxLineCount = lineCount;
    this.lines = new String[this.maxLineCount];
    this.lineSpacing = lineSpace;
    this.font = newfont;
  }

  public TextBox(PFont font, int lineCount, int lineSpace, Point loc, int bwidth){
    this("Console", font, lineCount, lineSpace, loc, bwidth);
  }

  /**
   * Draw this bitch!
   */
  public void draw(){
    super.draw();

    //draw the text, yo!
    textFont(this.font);
    fill(color(250));
    int gutterCorrection = (super.getHeight() / (2 * this.lines.length)) + gutter - 2;
    for(int i = 0; i < this.lines.length; i++){
      if(this.lines[i] != null){
        //draw the actual text and be sure to bump it down
        text(this.lines[i], super.getLocation().x, super.getLocation().y + (this.lineSpacing * i) + gutterCorrection); 
      }
    }
  }

  /**
   * Tick this shite!
   */
  public void tick(float delta){

  }

  /**
   * Adds a line to the text box, bottom up, whooooooo
   */
  public void addLine(String newLine){
    boolean pushlines = true;

    //first, see if there are already spots in the lines array
    for(int i = 0; i < this.lines.length; i++){
      if(this.lines[i] == null){
        //yep there is
        this.lines[i] = newLine; 
        pushlines = false;
        break;
      }
    }

    if(pushlines){
      //push the lines up a slot each
      for(int i = 0; i < this.lines.length - 1; i++){
        this.lines[i] = this.lines[i + 1];
      }

      //and tag on the new one
      this.lines[this.lines.length - 1] = newLine;
    }

    //done!
  }
}


/**
 * A slider UI element that allows you to slide a toggle between two values.
 * This is just a simple element, that captures the motion of the mouse
 * through the ticking system.
 */

class Slider extends UIElement{

  private float maxValue;
  private float minValue;
  private float sliderPosition;
  private int sliderWidth = 20;
  private PFont font;           //the font being used to draw with, this is used directly, without scaling

  /**
   * Default constructor
   */
  private Slider(PFont thefont, float newmin, float newmax, Point newloc, int bHeight){
    super("sliders", newloc, 20, bHeight);
    this.font = thefont;
    this.maxValue = newmax;
    this.minValue = newmin;
    this.sliderPosition = newmin;
  } 

  /**
   * Time to draw some stuff
   */
  public void draw(){
    super.draw();
    fill(color(255));
    text(int(this.sliderPosition), super.getLocation().x, super.getLocation().y + super.getHeight() - 10);
  }

  /**
   * Time to tick some things
   */
  public void tick(float delta){
    if(mousePressed){
      if(mouseX > super.getLocation().x && mouseX < super.getLocation().x + super.getWidth()){
        if(mouseY > super.getLocation().y && mouseY < super.getLocation().y + super.getHeight()){
          //okay, all inside the slider, lets get the new value;
          this.sliderPosition = (1 - (mouseY / (super.getLocation().y + super.getHeight()))) * (this.maxValue - this.minValue) + this.minValue;
        }
      }
    }
  }
}
